------------------------------------------------------------------------------
--
-- Implementation of the game of Hangman.  Hppy C@mper
--
-- Feel free to use in any way, but please give credits.
--
-- Say "help" for help.
--
------------------------------------------------------------------------------

hm_active   = false
hm_phrase   = nil
hm_image_id = nil
hm_faulty_letters  = ""
hm_correct_letters = ""

hm_hangman_command = "hangman"
hm_guess_command   = "guess"
hm_help_command    = "help"

hm_white = "255255255"
hm_red   = "255000000"

hm_max_faulty_letters = 10

hm_image_x = 36
hm_image_y = 180

hm_faulty_letters_hud_id = 48
hm_faulty_letters_x = 4
hm_faulty_letters_y = 110

hm_phrase_hud_id = 49
hm_phrase_x = 4
hm_phrase_y = 232


addhook("say", "hm_say")
function hm_say(id, message)
    local iterator = message:gmatch("[^%s]+")
    local command = iterator()

    if command == hm_hangman_command then
        return hm_hangman(id, iterator)
    elseif command == hm_guess_command then
        return hm_guess(id, iterator)
    elseif command == hm_help_command then
        msg2(id, hm_hangman_command .. " <phrase>  -- start a new game of Hangman")
        msg2(id, hm_guess_command .. " <letter>  -- guess a letter")

        return 1
    end
end

-- Display text for new players
addhook("team", "hm_team")
function hm_team(id, team, look)
    print("team "..team)
    if hm_active then
        hm_update_hangman()
    end
end

addhook("startround", "hm_startround")
function hm_startround(id, message)
    if hm_active then
        -- All images are removed at round start
        hm_draw_image()
    else
        hm_hudtxt(hm_faulty_letters_hud_id, '', 0, 0, 0)
        hm_hudtxt(hm_phrase_hud_id, '', 0, 0, 0)
    end
end

-- Start a new game of hangman
function hm_hangman(id, iterator)
    if hm_active then
        msg2(id, hm_red .. "Error: There is already an active hangman session.")
        msg2(id, hm_red .. "You must wait until it has finished before starting a new one.")
        return 1
    end

    local phrase = {}
    local word = iterator()

    while word ~= nil do
        if string.match(word, '[^%a]') ~= nil then
            msg2(id, hm_red .. "Error: Only letters a-z are allowed.")
            return 1
        end
        table.insert(phrase, word:lower())
        word = iterator()
    end

    hm_active = true
    hm_phrase = phrase
    hm_faulty_letters  = ""
    hm_correct_letters = ""

    msg(hm_white .. "New game of Hangman started by " .. player(id, "name") .. ".")

    hm_update_hangman(id)

    return 1
end

-- Guess a letter
function hm_guess(id, iterator)
    if not hm_active then
        msg2(id, hm_red .. "Error: There is no active Hangman session.")
        return 1
    end

    local guess = iterator()
    if guess == nil then
        msg2(id, hm_red .. "Error: Missing letter.")
        return 1
    end

    guess = guess:sub(1,1):lower()

    if string.match(guess, '[^%a]') ~= nil then
        msg2(id, hm_red .. "Error: Only letters a-z are allowed.")
        return 1
    end

    if string.match(hm_faulty_letters, guess) ~= nil or string.match(hm_correct_letters, guess) ~= nil then
        msg2(id, hm_red .. "Error: '" .. guess .. "' has already been used.")
        return 1
    end

    for _, word in ipairs(hm_phrase) do
        if word:match(guess) ~= nil then
            hm_correct_letters = hm_correct_letters .. guess
            hm_update_hangman(id)
            return 0
        end
    end

    hm_faulty_letters = hm_faulty_letters .. guess

    hm_update_hangman(id)

    return 0
end

function hm_hudtxt(id, text, x, y, align)
    parse("hudtxt " .. id .. ' "' .. text .. '" ' .. x .. " " .. y .. " " .. align)
end

-- Update images and HUD texts
function hm_update_hangman(id)
    hm_hudtxt(hm_faulty_letters_hud_id, hm_red .. hm_faulty_letters, hm_faulty_letters_x, hm_faulty_letters_y, 0)

    hm_draw_image()

    local completed = hm_display_phrase()

    if completed then
        msg(hm_white .. player(id, "name") .. " solved the phrase!!")
        hm_active = false
    elseif hm_faulty_letters:len() >= hm_max_faulty_letters then
        msg(hm_white .. "Game over.")
        hm_display_phrase(true)
        hm_active = false
    end
end

-- Display the phrase in the HUD
function hm_display_phrase(show_all)
    local completed = true
    local phrase_string = ""

    for _, word in ipairs(hm_phrase) do
        for i = 1, word:len() do
            local letter = word:sub(i, i)
            if show_all or string.match(hm_correct_letters, letter) ~= nil then
                phrase_string = phrase_string .. letter
            else
                phrase_string = phrase_string .. "."
                completed = false
            end
        end
        phrase_string = phrase_string .. " "
    end

    -- Trim spaces
    phrase_string = phrase_string:match'^%s*(.*%S)' or ''

    hm_hudtxt(hm_phrase_hud_id, hm_white .. phrase_string, hm_phrase_x, hm_phrase_y, 0)

    -- This is ugly, but I am too lazy to make a separate function
    return completed
end

-- Draw the hangman image
function hm_draw_image()
    if hm_image_id ~= nil then
        freeimage(hm_image_id)
    end

    hm_image_id = image("gfx/hangman/Hangman" .. hm_faulty_letters:len() + 1 .. ".png", hm_image_x, hm_image_y, 2)
end
